#define UNICODE
#define _UNICODE
#define _CRT_SECURE_NO_DEPRECATE
#include "LBParser.h"
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fstream>
#include <string>
#include <vector>
#include "getopt.h"
#pragma comment(lib,"LBParser.lib")

void usage()
{
    printf("Usage: HTML2LRF [-t \"Book title\"] [-a \"Book Author\"] [-o output_filename] [-r response_file] <url|path.html> [<url|path.html>...]");
}

std::wstring normalize_path(const std::wstring& path)
{
	if (path.find(L"://")!=path.npos || path.find(L":\\")!=path.npos)
		return path;
	wchar_t* full = _wfullpath(0, path.c_str(), 0);
	std::wstring newpath(full);
	free(full);
	return newpath;
}

void wmain(int argc, wchar_t* argv[])
{
  printf("HTML2LRF v 0.3 (c) 2006 Igor Skochinsky\n");
  std::wstring out_fname(L"test.lrf");
  std::wstring title(L"Test Book");
  std::wstring author(L"Unknown Author");
  std::wstring respfile;
  std::wstring design_file(argv[0]);
  int pos = design_file.find_last_of(L"\\/");
  design_file.resize((pos==design_file.npos)?0:pos+1);
  design_file+=L"DesignHorizontal.lrf";
  //wprintf(L"%s\n",design_file.c_str());

  int ch;
  while ((ch = getopt(argc, argv, L"ht:a:o:r:")) != -1) {
    switch (ch) {
    case 't':
      title = optarg;
      break;
    case 'a':
      author = optarg;
      break;
    case 'o':
      out_fname = optarg;
      break;
    case 'r':
    	respfile = optarg;
      break;
    case '?':
    case 'h':
    default:
      usage();
      return;
    }
  }
  argc -= optind;
  argv += optind;
  std::vector<std::wstring> input_files;
  std::vector<std::wstring> titles;
  if (respfile.length())
  {
  	std::wifstream fin(_wfopen(respfile.c_str(), L"rt"));
  	std::wstring line;
  	int pos;
  	while (getline(fin, line))
  	{
      //wprintf(L"%s\n",line.c_str());
  	  if (line.size()>1 && line[1]==L':')
  	    switch(line[0])
  	    {
  	      case L'A':
  	        author.assign(line,2,-1);
  	        break;
  	      case L'T':
  	        title.assign(line,2,-1);
  	        break;
  	      case L'O':
  	        out_fname.assign(line,2,-1);
  	        break;
  	      case L'P':
  	        pos = line.rfind(L'|');
  	        input_files.push_back(std::wstring(line, 2, pos-2));
  	        if (pos!=-1)
  	        	titles.push_back(std::wstring(line, pos+1,-1));
  	        break;
  	    }
  	}
  }
  else
  {
    while (argc--)
      input_files.push_back(*argv++);
  }
  if (input_files.size()==0)
  {
    usage();
    return;
  }

  char bookID[17];
  time_t now = time(0);
  strftime(bookID,16,"FBENNP%m%d%H%M%S",localtime(&now));
  bookID[16]=0;
  CLBParser parser;
  parser.SetBookTitle(title.c_str());
  parser.SetBookTitle_read(title.c_str());
  parser.SetBookAuthor(author.c_str());
  parser.SetBookAuthor_read(author.c_str());
  parser.SetBookID(bookID);
  parser.SetLRFFileName(out_fname.c_str());
  parser.SetDesignFileName(design_file.c_str());
  parser.CreateNewBook();
  //parser.AddPage("file://E:/Work/progs/Reader/LibrieToolbar/test/test.html",0,0);
  wprintf(L"Writing to %s...\n",out_fname.c_str());
  for (int i=0;i<input_files.size();i++)
  {
    wprintf(L"%s\n",input_files[i].c_str());
    if (i<titles.size())
    {
      //wprintf(L"\t%s\n",titles[i].c_str());
      parser.SetTocTitle(titles[i].c_str());
    }
    parser.AddPage(normalize_path(input_files[i]).c_str(),0,0);
  }
  printf("Finalizing...");
  parser.FinalizeBook();
  printf(" done.\n");
}
